@@ -1,66 +1,61 @@
+"""
+$description Live and on-demand video service from World Wrestling Entertainment, Inc.
+$url network.wwe.com
+$type live, vod
+$account Required
+"""
+
 import json
 import logging
 import re
 from urllib.parse import parse_qsl, urlparse
 
-from streamlink import PluginError
-from streamlink.plugin import Plugin, PluginArgument, PluginArguments
+from streamlink.plugin import Plugin, PluginError, pluginargument, pluginmatcher
 from streamlink.plugin.api import useragents
-from streamlink.stream import HLSStream
-from streamlink.utils import memoize
+from streamlink.stream.hls import HLSStream
 from streamlink.utils.times import seconds_to_hhmmss
 
+
 log = logging.getLogger(__name__)
 
 
+@pluginmatcher(re.compile(
+    r"https?://network\.wwe\.com/(?:video|live)/(?P<stream_id>\d+)",
+))
+@pluginargument(
+    "email",
+    required=True,
+    requires=["password"],
+    metavar="EMAIL",
+    help="The email associated with your WWE Network account, required to access any WWE Network stream.",
+)
+@pluginargument(
+    "password",
+    required=True,
+    sensitive=True,
+    metavar="PASSWORD",
+    help="A WWE Network account password to use with --wwenetwork-email.",
+)
 class WWENetwork(Plugin):
-    url_re = re.compile(r"https?://watch\.wwe\.com/(channel)?")
-    site_config_re = re.compile(r'''">window.__data = (\{.*?\})</script>''')
+    site_config_re = re.compile(r"""">window.__data = (\{.*?\})</script>""")
     stream_url = "https://dce-frontoffice.imggaming.com/api/v2/stream/{id}"
     live_url = "https://dce-frontoffice.imggaming.com/api/v2/event/live"
     login_url = "https://dce-frontoffice.imggaming.com/api/v2/login"
-    page_config_url = "https://cdn.watch.wwe.com/api/page"
+
     API_KEY = "cca51ea0-7837-40df-a055-75eb6347b2e7"
 
     customer_id = 16
-    arguments = PluginArguments(
-        PluginArgument(
-            "email",
-            required=True,
-            metavar="EMAIL",
-            requires=["password"],
-            help="""
-        The email associated with your WWE Network account,
-        required to access any WWE Network stream.
-        """
-        ),
-        PluginArgument(
-            "password",
-            sensitive=True,
-            metavar="PASSWORD",
-            help="""
-        A WWE Network account password to use with --wwenetwork-email.
-        """
-        )
-    )
 
-    def __init__(self, url):
-        super().__init__(url)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.session.http.headers.update({"User-Agent": useragents.CHROME})
         self.auth_token = None
 
-    @classmethod
-    def can_handle_url(cls, url):
-        return cls.url_re.match(url) is not None
-
-    def get_title(self):
-        return self.item_config['title']
-
     def request(self, method, url, **kwargs):
         headers = kwargs.pop("headers", {})
         headers.update({"x-api-key": self.API_KEY,
-                        "Origin": "https://watch.wwe.com",
-                        "Referer": "https://watch.wwe.com/signin",
+                        "Origin": "https://network.wwe.com",
+                        "Referer": "https://network.wwe.com/signin",
                         "Accept": "application/json",
                         "Realm": "dce.wwe"})
         if self.auth_token:
@@ -75,14 +70,14 @@ class WWENetwork(Plugin):
             log.debug("API request failed: {0}:{1} ({2})".format(
                 data["status"],
                 data.get("code"),
-                "; ".join(data.get("messages", []))
+                "; ".join(data.get("messages", [])),
             ))
         return data
 
     def login(self, email, password):
         log.debug("Attempting login as {0}".format(email))
         # sets some required cookies to login
-        data = self.request('POST', self.login_url,
+        data = self.request("POST", self.login_url,
                             data=json.dumps({"id": email, "secret": password}),
                             headers={"Content-Type": "application/json"})
         if "authorisationToken" in data:
@@ -90,51 +85,28 @@ class WWENetwork(Plugin):
 
         return self.auth_token
 
-    @property
-    @memoize
-    def item_config(self):
-        log.debug("Loading page config")
-        p = urlparse(self.url)
-        res = self.session.http.get(self.page_config_url,
-                                    params=dict(device="web_browser",
-                                                ff="idp,ldp",
-                                                item_detail_expand="all",
-                                                lang="en-US",
-                                                list_page_size="1",
-                                                max_list_prefetch="1",
-                                                path=p.path,
-                                                segments="es",
-                                                sub="Registered",
-                                                text_entry_format="html"))
-        data = self.session.http.json(res)
-        return data["item"]
-
     def _get_media_info(self, content_id):
         """
         Get the info about the content, based on the ID
         :param content_id: contentId for the video
         :return:
         """
-        info = self.request('GET', self.stream_url.format(id=content_id))
-        return self.request('GET', info.get("playerUrlCallback"))
+        info = self.request("GET", self.stream_url.format(id=content_id))
+        return self.request("GET", info.get("playerUrlCallback"))
 
-    def _get_video_id(self):
-        #  check the page to find the contentId
-        log.debug("Searching for content ID")
-        try:
-            if self.item_config['type'] == "channel":
-                return self._get_live_id()
-            else:
-                return "vod/{id}".format(id=self.item_config['customFields']['DiceVideoId'])
-        except KeyError:
-            log.error("Could not find video ID")
-            return
+    def _get_video_id(self, stream_id):
+        live_id = self._get_live_id(stream_id)
+        if not live_id:
+            return "vod/{0}".format(stream_id)
 
-    def _get_live_id(self):
+        return live_id
+
+    def _get_live_id(self, stream_id):
         log.debug("Loading live event")
-        res = self.request('GET', self.live_url)
-        for event in res.get('events', []):
-            return "event/{sportId}/{propertyId}/{tournamentId}/{id}".format(**event)
+        res = self.request("GET", self.live_url)
+        for event in res.get("events", []):
+            if str(event["id"]) == stream_id:
+                return "event/{sportId}/{propertyId}/{tournamentId}/{id}".format(**event)
 
     def _get_streams(self):
         if not self.login(self.get_option("email"), self.get_option("password")):
@@ -147,7 +119,8 @@ class WWENetwork(Plugin):
         except ValueError:
             start_point = 0
 
-        content_id = self._get_video_id()
+        stream_id = self.match.group("stream_id")
+        content_id = self._get_video_id(stream_id)
 
         if content_id:
             log.debug("Found content ID: {0}".format(content_id))
@@ -156,7 +129,7 @@ class WWENetwork(Plugin):
                 yield from HLSStream.parse_variant_playlist(
                     self.session,
                     info["hlsUrl"],
-                    start_offset=start_point
+                    start_offset=start_point,
                 ).items()
             else:
                 log.error("Could not find the HLS URL")
