@@ -1,92 +1,93 @@
+"""
+$description American live TV channel and video on-demand service owned by Warner Bros.
+$url adultswim.com
+$type live, vod
+$region various
+$notes VODs may be protected by DRM
+"""
+
 import logging
 import re
+from urllib.parse import urlparse, urlunparse
 
-from streamlink.compat import urlparse, urlunparse
-from streamlink.exceptions import PluginError
-from streamlink.plugin import Plugin
+from streamlink.plugin import Plugin, PluginError, pluginmatcher
 from streamlink.plugin.api import validate
-from streamlink.stream import HLSStream
-from streamlink.utils import parse_json
+from streamlink.stream.hls import HLSStream
+from streamlink.utils.parse import parse_json
+
 
 log = logging.getLogger(__name__)
 
 
+@pluginmatcher(re.compile(r"""
+    https?://(?:www\.)?adultswim\.com
+    /(streams|videos)
+    (?:/([^/]+))?
+    (?:/([^/]+))?
+""", re.VERBOSE))
 class AdultSwim(Plugin):
-    token_url = 'https://token.ngtv.io/token/token_spe'
-    video_data_url = 'https://www.adultswim.com/api/shows/v1/media/{0}/desktop'
+    token_url = "https://token.ngtv.io/token/token_spe"
+    video_data_url = "https://www.adultswim.com/api/shows/v1/media/{0}/desktop"
 
     app_id_js_url_re = re.compile(
-        r'''<script src="([^"]*asvp\..*?\.bundle\.js)">'''
+        r"""<script src="([^"]*asvp\..*?\.bundle\.js)">""",
     )
 
     app_id_re = re.compile(
-        r'''CDN_TOKEN_APP_ID="(.*?)"'''
+        r'''CDN_TOKEN_APP_ID="(.*?)"''',
     )
 
     json_data_re = re.compile(
-        r'''<script id="__NEXT_DATA__" type="application/json">({.*})</script>'''
+        r"""<script id="__NEXT_DATA__" type="application/json">({.*})</script>""",
     )
 
-    truncate_url_re = re.compile(r'''(.*)/\w+/?''')
-
-    url_re = re.compile(
-        r'''https?://(?:www\.)?adultswim\.com
-        /(streams|videos)
-        (?:/([^/]+))?
-        (?:/([^/]+))?
-        ''', re.VERBOSE,
-    )
+    truncate_url_re = re.compile(r"""(.*)/\w+/?""")
 
     _api_schema = validate.Schema({
-        'media': {
-            'desktop': {
-                validate.text: {
-                    'url': validate.url()
-                }
-            }
+        "media": {
+            "desktop": {
+                str: {
+                    "url": validate.url(),
+                },
+            },
         }},
-        validate.get('media'),
-        validate.get('desktop'),
-        validate.filter(lambda k, v: k in ['unprotected', 'bulkaes'])
+        validate.get("media"),
+        validate.get("desktop"),
+        validate.filter(lambda k, v: k in ["unprotected", "bulkaes"]),
     )
 
     _stream_data_schema = validate.Schema({
-        'props': {'__REDUX_STATE__': {'streams': [{
-            'id': validate.text,
-            'stream': validate.text,
+        "props": {"__REDUX_STATE__": {"streams": [{
+            "id": str,
+            "stream": str,
         }]}}},
-        validate.get('props'),
-        validate.get('__REDUX_STATE__'),
-        validate.get('streams'),
+        validate.get("props"),
+        validate.get("__REDUX_STATE__"),
+        validate.get("streams"),
     )
 
     _token_schema = validate.Schema(
         validate.any(
-            {'auth': {'token': validate.text}},
-            {'auth': {'error': {'message': validate.text}}},
+            {"auth": {"token": str}},
+            {"auth": {"error": {"message": str}}},
         ),
-        validate.get('auth'),
+        validate.get("auth"),
     )
 
     _video_data_schema = validate.Schema({
-        'props': {'pageProps': {'__APOLLO_STATE__': {
-            validate.text: {
-                validate.optional('id'): validate.text,
-                validate.optional('slug'): validate.text,
-            }
+        "props": {"pageProps": {"__APOLLO_STATE__": {
+            str: {
+                validate.optional("id"): str,
+                validate.optional("slug"): str,
+            },
         }}}},
-        validate.get('props'),
-        validate.get('pageProps'),
-        validate.get('__APOLLO_STATE__'),
-        validate.filter(lambda k, v: k.startswith('Video:')),
+        validate.get("props"),
+        validate.get("pageProps"),
+        validate.get("__APOLLO_STATE__"),
+        validate.filter(lambda k, v: k.startswith("Video:")),
     )
 
-    @classmethod
-    def can_handle_url(cls, url):
-        match = AdultSwim.url_re.match(url)
-        return match is not None
-
-    def _get_stream_data(self, id):
+    def _get_stream_data(self, streamid):
         res = self.session.http.get(self.url)
         m = self.json_data_re.search(res.text)
         if m and m.group(1):
@@ -95,9 +96,8 @@ class AdultSwim(Plugin):
             raise PluginError("Failed to get json_data")
 
         for stream in streams:
-            if 'id' in stream:
-                if id == stream['id'] and 'stream' in stream:
-                    return stream['stream']
+            if "id" in stream and streamid == stream["id"] and "stream" in stream:
+                return stream["stream"]
 
     def _get_video_data(self, slug):
         m = self.truncate_url_re.search(self.url)
@@ -114,9 +114,9 @@ class AdultSwim(Plugin):
             raise PluginError("Failed to get json_data")
 
         for video in videos:
-            if 'slug' in videos[video]:
-                if slug == videos[video]['slug'] and 'id' in videos[video]:
-                    return videos[video]['id']
+            if "slug" in videos[video]:
+                if slug == videos[video]["slug"] and "id" in videos[video]:
+                    return videos[video]["id"]
 
     def _get_token(self, path):
         res = self.session.http.get(self.url)
@@ -134,23 +134,22 @@ class AdultSwim(Plugin):
         log.debug("app_id={0}".format(app_id))
 
         res = self.session.http.get(self.token_url, params=dict(
-            format='json',
+            format="json",
             appId=app_id,
             path=path,
         ))
 
         token_data = self.session.http.json(res, schema=self._token_schema)
-        if 'error' in token_data:
-            raise PluginError(token_data['error']['message'])
+        if "error" in token_data:
+            raise PluginError(token_data["error"]["message"])
 
-        return token_data['token']
+        return token_data["token"]
 
     def _get_streams(self):
-        url_match = self.url_re.match(self.url)
-        url_type, show_name, episode_name = url_match.groups()
+        url_type, show_name, episode_name = self.match.groups()
 
-        if url_type == 'streams' and not show_name:
-            url_type = 'live-stream'
+        if url_type == "streams" and not show_name:
+            url_type = "live-stream"
         elif not show_name:
             raise PluginError("Missing show_name for url_type: {0}".format(
                 url_type,
@@ -158,16 +157,16 @@ class AdultSwim(Plugin):
 
         log.debug("URL type={0}".format(url_type))
 
-        if url_type == 'live-stream':
+        if url_type == "live-stream":
             video_id = self._get_stream_data(url_type)
-        elif url_type == 'streams':
+        elif url_type == "streams":
             video_id = self._get_stream_data(show_name)
-        elif url_type == 'videos':
+        elif url_type == "videos":
             if show_name is None or episode_name is None:
                 raise PluginError(
                     "Missing show_name or episode_name for url_type: {0}".format(
                         url_type,
-                    )
+                    ),
                 )
             video_id = self._get_video_data(episode_name)
         else:
@@ -180,17 +179,17 @@ class AdultSwim(Plugin):
         res = self.session.http.get(self.video_data_url.format(video_id))
 
         url_data = self.session.http.json(res, schema=self._api_schema)
-        if 'unprotected' in url_data:
-            url = url_data['unprotected']['url']
-        elif 'bulkaes' in url_data:
-            url_parsed = urlparse(url_data['bulkaes']['url'])
+        if "unprotected" in url_data:
+            url = url_data["unprotected"]["url"]
+        elif "bulkaes" in url_data:
+            url_parsed = urlparse(url_data["bulkaes"]["url"])
             token = self._get_token(url_parsed.path)
             url = urlunparse((
                 url_parsed.scheme,
                 url_parsed.netloc,
                 url_parsed.path,
                 url_parsed.params,
-                "{0}={1}".format('hdnts', token),
+                "{0}={1}".format("hdnts", token),
                 url_parsed.fragment,
             ))
         else:
