@@ -1,17 +1,23 @@
+"""
+$description TV and live video game broadcasts, artist performances and personal daily-life video blogs & shows.
+$url play.afreecatv.com
+$type live
+"""
+
 import logging
 import re
 
-from streamlink.plugin import Plugin, PluginArgument, PluginArguments
+from streamlink.plugin import Plugin, pluginargument, pluginmatcher
 from streamlink.plugin.api import validate
-from streamlink.stream import HLSStream
-from streamlink.stream.hls import HLSStreamReader, HLSStreamWriter
+from streamlink.stream.hls import HLSStream, HLSStreamReader, HLSStreamWriter
+
 
 log = logging.getLogger(__name__)
 
 
 class AfreecaHLSStreamWriter(HLSStreamWriter):
-    def should_filter_sequence(self, sequence):
-        return "preloading" in sequence.segment.uri or super().should_filter_sequence(sequence)
+    def should_filter_segment(self, segment):
+        return "preloading" in segment.uri or super().should_filter_segment(segment)
 
 
 class AfreecaHLSStreamReader(HLSStreamReader):
@@ -19,15 +25,32 @@ class AfreecaHLSStreamReader(HLSStreamReader):
 
 
 class AfreecaHLSStream(HLSStream):
-    def open(self):
-        reader = AfreecaHLSStreamReader(self)
-        reader.open()
-        return reader
+    __reader__ = AfreecaHLSStreamReader
 
 
+@pluginmatcher(re.compile(
+    r"https?://play\.afreecatv\.com/(?P<username>\w+)(?:/(?P<bno>:\d+))?",
+))
+@pluginargument(
+    "username",
+    sensitive=True,
+    requires=["password"],
+    metavar="USERNAME",
+    help="The username used to register with afreecatv.com.",
+)
+@pluginargument(
+    "password",
+    sensitive=True,
+    metavar="PASSWORD",
+    help="A afreecatv.com account password to use with --afreeca-username.",
+)
+@pluginargument(
+    "purge-credentials",
+    action="store_true",
+    help="Purge cached AfreecaTV credentials to initiate a new session and reauthenticate.",
+)
 class AfreecaTV(Plugin):
     _re_bno = re.compile(r"var nBroadNo = (?P<bno>\d+);")
-    _re_url = re.compile(r"https?://play\.afreecatv\.com/(?P<username>\w+)(?:/(?P<bno>:\d+))?")
 
     CHANNEL_API_URL = "http://live.afreecatv.com/afreeca/player_live_api.php"
     CHANNEL_RESULT_OK = 1
@@ -47,44 +70,21 @@ class AfreecaTV(Plugin):
                 validate.optional("RMD"): str,
                 validate.optional("AID"): str,
                 validate.optional("CDN"): str,
-            }
+            },
         },
-        validate.get("CHANNEL")
+        validate.get("CHANNEL"),
     )
     _schema_stream = validate.Schema(
         {
             validate.optional("view_url"): validate.url(
-                scheme=validate.any("rtmp", "http")
+                scheme=validate.any("rtmp", "http"),
             ),
             "stream_status": str,
-        }
+        },
     )
 
-    arguments = PluginArguments(
-        PluginArgument(
-            "username",
-            sensitive=True,
-            requires=["password"],
-            metavar="USERNAME",
-            help="The username used to register with afreecatv.com."
-        ),
-        PluginArgument(
-            "password",
-            sensitive=True,
-            metavar="PASSWORD",
-            help="A afreecatv.com account password to use with --afreeca-username."
-        ),
-        PluginArgument(
-            "purge-credentials",
-            action="store_true",
-            help="""
-        Purge cached AfreecaTV credentials to initiate a new session
-        and reauthenticate.
-        """),
-    )
-
-    def __init__(self, url):
-        super().__init__(url)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self._authed = (
             self.session.http.cookies.get("PdboxBbs")
             and self.session.http.cookies.get("PdboxSaveTicket")
@@ -93,10 +93,6 @@ class AfreecaTV(Plugin):
             and self.session.http.cookies.get("RDB")
         )
 
-    @classmethod
-    def can_handle_url(cls, url):
-        return cls._re_url.match(url) is not None
-
     @classmethod
     def stream_weight(cls, key):
         weight = cls.QUALITY_WEIGHTS.get(key)
@@ -109,8 +105,11 @@ class AfreecaTV(Plugin):
         data = {
             "bid": username,
             "bno": broadcast,
+            "from_api": "0",
             "mode": "landing",
             "player_type": "html5",
+            "pwd": "",
+            "stream_type": "common",
             "type": "live",
         }
         res = self.session.http.post(self.CHANNEL_API_URL, data=data)
@@ -120,29 +119,33 @@ class AfreecaTV(Plugin):
         data = {
             "bid": username,
             "bno": broadcast,
+            "from_api": "0",
+            "mode": "landing",
+            "player_type": "html5",
             "pwd": "",
             "quality": quality,
-            "type": "pwd"
+            "stream_type": "common",
+            "type": "aid",
         }
         res = self.session.http.post(self.CHANNEL_API_URL, data=data)
         return self.session.http.json(res, schema=self._schema_channel)
 
-    def _get_stream_info(self, broadcast, quality, cdn, rmd):
+    def _get_stream_info(self, broadcast, quality, rmd):
         params = {
-            "return_type": cdn,
-            "broad_key": f"{broadcast}-flash-{quality}-hls",
+            "return_type": "gs_cdn_pc_web",
+            "broad_key": f"{broadcast}-common-{quality}-hls",
         }
         res = self.session.http.get(f"{rmd}/broad_stream_assign.html", params=params)
         return self.session.http.json(res, schema=self._schema_stream)
 
-    def _get_hls_stream(self, broadcast, username, quality, cdn, rmd):
+    def _get_hls_stream(self, broadcast, username, quality, rmd):
         keyjson = self._get_hls_key(broadcast, username, quality)
 
         if keyjson["RESULT"] != self.CHANNEL_RESULT_OK:
             return
         key = keyjson["AID"]
 
-        info = self._get_stream_info(broadcast, quality, cdn, rmd)
+        info = self._get_stream_info(broadcast, quality, rmd)
 
         if "view_url" in info:
             return AfreecaHLSStream(self.session, info["view_url"], params={"aid": key})
@@ -161,17 +164,16 @@ class AfreecaTV(Plugin):
         res = self.session.http.post("https://login.afreecatv.com/app/LoginAction.php", data=data)
         data = self.session.http.json(res)
         log.trace(f"{data!r}")
-        if data["RESULT"] == self.CHANNEL_RESULT_OK:
-            self.save_cookies()
-            return True
-        else:
+        if data["RESULT"] != self.CHANNEL_RESULT_OK:
             return False
+        self.save_cookies()
+        return True
 
     def _get_streams(self):
         login_username = self.get_option("username")
         login_password = self.get_option("password")
 
-        self.session.http.headers.update({"Referer": self.url})
+        self.session.http.headers.update({"Referer": self.url, "Origin": "http://play.afreecatv.com"})
 
         if self.options.get("purge_credentials"):
             self.clear_cookies()
@@ -187,7 +189,7 @@ class AfreecaTV(Plugin):
             else:
                 log.error("Failed to login")
 
-        m = self._re_url.match(self.url).groupdict()
+        m = self.match.groupdict()
         username = m["username"]
         bno = m["bno"]
         if bno is None:
@@ -209,12 +211,12 @@ class AfreecaTV(Plugin):
         elif channel.get("RESULT") != self.CHANNEL_RESULT_OK:
             return
 
-        (broadcast, rmd, cdn) = (channel["BNO"], channel["RMD"], channel["CDN"])
-        if not (broadcast and rmd and cdn):
+        (broadcast, rmd) = (channel["BNO"], channel["RMD"])
+        if not (broadcast and rmd):
             return
 
         for qkey in self.QUALITYS:
-            hls_stream = self._get_hls_stream(broadcast, username, qkey, cdn, rmd)
+            hls_stream = self._get_hls_stream(broadcast, username, qkey, rmd)
             if hls_stream:
                 yield qkey, hls_stream
 
