@@ -1,58 +1,53 @@
+"""
+$description Global live-streaming and video hosting social platform.
+$url facebook.com
+$type live, vod
+$metadata title
+"""
+
 import logging
 import re
-from html import unescape as html_unescape
 from urllib.parse import unquote_plus, urlencode
 
-from streamlink.plugin import Plugin
-from streamlink.plugin.api.utils import itertags
-from streamlink.stream import DASHStream, HTTPStream
-from streamlink.utils import parse_json
+from streamlink.plugin import Plugin, pluginmatcher
+from streamlink.plugin.api import validate
+from streamlink.stream.dash import DASHStream
+from streamlink.stream.http import HTTPStream
+from streamlink.utils.parse import parse_json
+
 
 log = logging.getLogger(__name__)
 
 
+@pluginmatcher(re.compile(r"""
+    https?://(?:www\.)?facebook
+    (?:\.com|wkhpilnemxj7asaniu7vnjjbiltxjqhye3mhbshg7kx5tfyd\.onion)
+    /[^/]+/(?:posts|videos)/(?P<video_id>\d+)
+""", re.VERBOSE))
 class Facebook(Plugin):
-    _url_re = re.compile(r'''(?x)https?://(?:www\.)?facebook(?:\.com|corewwwi.onion)
-        /[^/]+/(?:posts|videos)/(?P<video_id>[0-9]+)''')
-    _src_re = re.compile(r'''(sd|hd)_src["']?\s*:\s*(?P<quote>["'])(?P<url>.+?)(?P=quote)''')
-    _dash_manifest_re = re.compile(r'''dash_manifest["']?\s*:\s*["'](?P<manifest>.+?)["'],''')
-    _playlist_re = re.compile(r'''video:\[({url:".+?}\])''')
+    _src_re = re.compile(r"""(sd|hd)_src["']?\s*:\s*(?P<quote>["'])(?P<url>.+?)(?P=quote)""")
+    _dash_manifest_re = re.compile(r"""dash_manifest["']?\s*:\s*["'](?P<manifest>.+?)["'],""")
+    _playlist_re = re.compile(r"""video:\[({url:".+?}\])""")
     _plurl_re = re.compile(r'''url:"(.*?)"''')
-    _pc_re = re.compile(r'''pkg_cohort["']\s*:\s*["'](.+?)["']''')
-    _rev_re = re.compile(r'''client_revision["']\s*:\s*(\d+),''')
-    _dtsg_re = re.compile(r'''DTSGInitialData["'],\s*\[\],\s*{\s*["']token["']\s*:\s*["'](.+?)["']''')
-    _title_re = re.compile(r'<meta property="og:title" content="([^\"]+)"')
+    _pc_re = re.compile(r"""pkg_cohort["']\s*:\s*["'](.+?)["']""")
+    _rev_re = re.compile(r"""client_revision["']\s*:\s*(\d+),""")
+    _dtsg_re = re.compile(r"""DTSGInitialData["'],\s*\[\],\s*{\s*["']token["']\s*:\s*["'](.+?)["']""")
     _DEFAULT_PC = "PHASED:DEFAULT"
     _DEFAULT_REV = 4681796
     _TAHOE_URL = "https://www.facebook.com/video/tahoe/async/{0}/?chain=true&isvideo=true&payloadtype=primary"
 
-    @classmethod
-    def can_handle_url(cls, url):
-        return cls._url_re.match(url) is not None
-
-    def get_title(self):
-        res = self.session.http.get(self.url)
-        m = self._title_re.search(res.text)
-        if m:
-            return html_unescape(m.group(1))
-
     def _parse_streams(self, res):
-        _found_stream_url = False
-        for meta in itertags(res.text, "meta"):
-            if meta.attributes.get("property") == "og:video:url":
-                stream_url = html_unescape(meta.attributes.get("content"))
-                if ".mpd" in stream_url:
-                    for s in DASHStream.parse_manifest(self.session, stream_url).items():
-                        yield s
-                        _found_stream_url = True
-                elif ".mp4" in stream_url:
-                    yield "vod", HTTPStream(self.session, stream_url)
-                    _found_stream_url = True
-                break
-        else:
+        stream_url = validate.Schema(
+            validate.parse_html(),
+            validate.xml_xpath_string(".//head/meta[@property='og:video:url'][@content][1]/@content"),
+        ).validate(res.text)
+        if not stream_url:
             log.debug("No meta og:video:url")
-
-        if _found_stream_url:
+        elif ".mpd" in stream_url:
+            yield from DASHStream.parse_manifest(self.session, stream_url).items()
+            return
+        elif ".mp4" in stream_url:
+            yield "vod", HTTPStream(self.session, stream_url)
             return
 
         for match in self._src_re.finditer(res.text):
@@ -80,9 +75,24 @@ class Facebook(Plugin):
 
     def _get_streams(self):
         self.session.set_option("ffmpeg-start-at-zero", True)
+        self.session.http.headers.update({"Accept-Language": "en-US"})
 
         done = False
         res = self.session.http.get(self.url)
+        log.trace(f"{res.url}")
+
+        title, canonical, self.title = validate.Schema(
+            validate.parse_html(),
+            validate.union((
+                validate.xml_xpath_string(".//head/title[1]/text()"),
+                validate.xml_xpath_string(".//head/meta[@res='canonical'][@href][1]/@href"),
+                validate.xml_xpath_string(".//head/meta[@property='og:title'][@content][1]/@content"),
+            )),
+        ).validate(res.text)
+        if canonical == "https://www.facebook.com/login/" or "log in" in title.lower():
+            log.error("This URL requires a login or may be accessible from a different IP address.")
+            return
+
         for s in self._parse_streams(res):
             done = True
             yield s
@@ -102,7 +112,7 @@ class Facebook(Plugin):
 
         # fallback to tahoe player url
         log.debug("Falling back to tahoe player")
-        video_id = self._url_re.match(self.url).group("video_id")
+        video_id = self.match.group("video_id")
         url = self._TAHOE_URL.format(video_id)
         data = {
             "__a": 1,
@@ -122,11 +132,10 @@ class Facebook(Plugin):
         res = self.session.http.post(
             url,
             headers={"Content-Type": "application/x-www-form-urlencoded"},
-            data=urlencode(data).encode("ascii")
+            data=urlencode(data).encode("ascii"),
         )
 
-        for s in self._parse_streams(res):
-            yield s
+        yield from self._parse_streams(res)
 
 
 __plugin__ = Facebook
