@@ -1,22 +1,26 @@
+"""
+$description Turkish live TV channel owned by Galatasaray TV.
+$url galatasaray.com
+$type live
+"""
+
 import logging
 import re
 
-from streamlink.plugin import Plugin
-from streamlink.stream import HLSStream
+from streamlink.plugin import Plugin, pluginmatcher
+from streamlink.stream.hls import HLSStream
+
 
 log = logging.getLogger(__name__)
 
 
+@pluginmatcher(re.compile(
+    r"https?://(?:www\.)?galatasaray\.com",
+))
 class GalatasarayTV(Plugin):
-    url_re = re.compile(r"https?://(?:www\.)?galatasaray\.com")
     playervars_re = re.compile(r"sources\s*:\s*\[\s*\{\s*type\s*:\s*\"(.*?)\",\s*src\s*:\s*\"(.*?)\"", re.DOTALL)
 
-    @classmethod
-    def can_handle_url(cls, url):
-        return cls.url_re.match(url) is not None
-
-    def get_title(self):
-        return "Galatasaray TV"
+    title = "Galatasaray TV"
 
     def _get_streams(self):
         res = self.session.http.get(self.url)
