@@ -1,24 +1,29 @@
+"""
+$description Live streaming news channel owned by Now/PCCW Group, based in Hong Kong.
+$url news.now.com
+$type live
+"""
+
 import json
 import logging
 import re
 
-from streamlink.plugin import Plugin
-from streamlink.stream import HLSStream
+from streamlink.plugin import Plugin, pluginmatcher
+from streamlink.stream.hls import HLSStream
+
 
 log = logging.getLogger(__name__)
 
 
+@pluginmatcher(re.compile(
+    r"https?://news\.now\.com/home/live",
+))
 class NowNews(Plugin):
-    _url_re = re.compile(r"https?://news\.now\.com/home/live")
-    epg_re = re.compile(r'''epg.getEPG\("(\d+)"\);''')
+    epg_re = re.compile(r"""epg.getEPG\("(\d+)"\);""")
     api_url = "https://hkt-mobile-api.nowtv.now.com/09/1/getLiveURL"
     backup_332_api = "https://d7lz7jwg8uwgn.cloudfront.net/apps_resource/news/live.json"
     backup_332_stream = "https://d3i3yn6xwv1jpw.cloudfront.net/live/now332/playlist.m3u8"
 
-    @classmethod
-    def can_handle_url(cls, url):
-        return cls._url_re.match(url) is not None
-
     def _get_streams(self):
         res = self.session.http.get(self.url)
         m = self.epg_re.search(res.text)
@@ -35,7 +40,7 @@ class NowNews(Plugin):
                     return HLSStream.parse_variant_playlist(self.session, self.backup_332_stream)
 
             api_res = self.session.http.post(self.api_url,
-                                             headers={"Content-Type": 'application/json'},
+                                             headers={"Content-Type": "application/json"},
                                              data=json.dumps(dict(channelno=channel_id,
                                                                   mode="prod",
                                                                   audioCode="",
